<template>
<!-- 回退面板 -->
  <div class="process-panel" @click="hiddenSuggestbox">
    <!-- 主体（上边） -->
    <div class="nodeName">
      <div class="list-data">
        <span class="title">{{$t('submitTo')}}</span>
        <span class="use-tip" @click.stop="hiddenSuggest()">
            {{$t('commonOpinions')}}
            <i class="fa fa-caret-down common-opinion-icon"></i>
        </span>
      </div>
    </div>
    <div class="nodeName" @click="designatedApprover">
      <!-- 流程选择 -->
      <mt-cell :title="$t('backTo')"  is-link>
        <span>{{selectedPerson.value?selectedPerson.value:$t('please_choose')}}</span>
      </mt-cell>
    </div>
    <div class="comment-box">
      <div class="wrap-left">
        <!-- 意见 -->
        <div>
          <mt-field 
            class="opinion"
            v-show="signMode == 'TEXT'" 
            :placeholder="$t('fillComments')"
            maxlength="140"
            type="textarea" 
            rows="5" 
            v-model="attitude">
          </mt-field>

          <!--手签-->
          <div class="idea-canvas"  v-show="signMode == 'SIGN'">
            <canvas
              id="canvas"
              :width="canvasWidth"
              :height="canvasHeight"
              style="background:rgba(246,246,246,1);"
              @touchstart='touchStart' 
              @touchmove='touchMove' 
              @touchend='touchEnd'
              ref="canvas"
            ></canvas>
          </div>
        </div>
      </div>
      <!-- 工具操作（右边） -->
      <div class="wrap-right">
        <span @click="changeSignMode" class="sign">
          {{$t('signature')}}
        </span>
        <span v-if="signMode == 'SIGN'" class="sign primary" @click="clear" size="small">重写</span>
        <span @click="onSubmit(act.workFlowType)" class="sign normal">
          {{$t('confirm')}}
        </span>
      </div>

      <!-- 常用意见操作 -->
      <div class="use-suggest" @click.stop v-if="suggestBoxVisible" style="">
        <div class="sug-data" v-if="allSuggests">
          <div class="sug-box">
            <span
              class="data-row"
              v-for="(item, index) in allSuggests"
              :key="index"
              @click="putData(item.content)"
            >
              <span :class="[showDelAndEdit?'suggest-name-one':'suggest-name-two']">{{item.content}}</span>
              <span class="icon" v-if="showDelAndEdit">
                <span>
                  <i class="fa fa-trash-o" @click="deleteSuggestion(item.id)"></i>
                </span>
                <span class="icon-two">
                  <i
                    class="fa fa-pencil-square-o"
                    @click.stop.prevent="editSuggestion(item.id, item.content)"
                  ></i>
                </span>
              </span>
            </span>
          </div>
        </div>
        <div class="sug-btn" v-if="!showInput">
          <div class="btns">
            <span @click="showInputBox">
              <img src="../assets/images/add.png">
              <span>{{$t('addCommonComments')}}</span>
            </span>
            <span @click="editSuggest">
              <img src="../assets/images/jump.png">
              <span>{{$t('editComment')}}</span>
            </span>
          </div>
        </div>
        <div class="sug-new" v-if="showEditFirst">
          <div class="new-input">
            <input type="text" v-model="input" />
          </div>
          <div class="keep-cancel">
            <span @click="cancelAddSuggest(1)">{{$t('cancel')}}</span>
            <span @click="addSuggestion">{{$t('verify')}}</span>
          </div>
        </div>
        <div class="sug-new" v-if="showEditSecond">
          <div class="new-input">
            <input type="text" v-model="inputSecond" />
          </div>
          <div class="keep-cancel">
            <span @click="cancelAddSuggest(2)">{{$t('cancel')}}</span>
            <span @click="updateSuggestion">{{$t('verify')}}</span>
          </div>
        </div>
      </div>
    </div>
  </div>
</template>

<script>
import { Toast } from 'mint-ui';
import API from "@/api.js";
export default {
  props: [
    "openParams",
    "stateId", 
    "act", 
    "selectedPerson",
    "approvers",

  ],
  inject: ['getBuildFormData'],
  data: function() {
    return {
      nextNodeIds: undefined,
      isSplit: true,
      signMode: "TEXT",//意见填写模式，SIGN:手写 TEXT：文本输入
      suggestBoxVisible: false,
      attitude: "",
      input: "", //单选框数据

      showInput: false, //是否展示添加常见意见框 开始false
      showEditFirst: false, //展示编辑意见框 


      showDelAndEdit: false,  //开始false
      allSuggests: [],
      showEditSecond: false,  //开始false
      inputSecond: "",
      opinionId: "", //更新意见信息的ID
      flowId: "", //当前流程ID
      currentNodeId: "",
      existCanvas: false, //是否存在手写签名


      //下拉选择框的数据
      popupVisible:false,
      canvasHeight: window.innerHeight*0.6 - 88 - 42, //弹框占屏幕60% 88为提交至和回退至的高度 42为手签重写确认按钮的高度
      canvasWidth: window.innerWidth,
    };
  },
  created() {
    this.getSuggest();
    this.getBackPanel();
  },

  mounted() {
    // 准备手写签名画布
    this.canvas = this.$refs.canvas; //指定canvas
    this.ctx = this.canvas.getContext("2d"); //设置2D渲染区域
    this.ctx.fillStyle = "#fff";
    this.ctx.fillRect(0, 0, 500, 142);
    //this.ctx.lineWidth = 1;//设置线的宽度
  },
  updated(){
    this.resizeCanvas();
  },
  methods: {
    //指定审批人
    
    designatedApprover() {
      this.backNodes.forEach(item => {
        item.value = item.name;
      });
      let slot = [];
      let value = {
        values: this.backNodes
      }; 
      slot.push(value);
      // slot.push(value);
      // allData.slots = slot;
      //this.$parent.$parent.$parent.$parent.$parent.flowRollback(slot);
      this.$emit("action", slot);
    },

    resizeCanvas(){
      this.canvas.width = window.innerWidth;
    },

    //清除画布数据
    clear() {
      this.ctx.clearRect(0, 0, this.ctx.canvas.width, this.ctx.canvas.height);
      this.existCanvas = false;
    },

    touchStart(ev) {
      ev = ev || event
      ev.preventDefault()
      this.canvasMoveUse = true;
      let rect = this.$refs.canvas.getBoundingClientRect();
      if (ev.touches.length == 1) {
        let obj = {
          x: ev.targetTouches[0].clientX,
          y: ev.targetTouches[0].clientY,
        }
        const canvasX = obj.x - this.$refs.canvas.offsetLeft;
        const canvasY = obj.y - rect.top;
        this.ctx.beginPath();  // 移动的起点
        this.ctx.moveTo(canvasX, canvasY);
      }
    },

    touchMove(ev) {
      if (this.canvasMoveUse) {
        // 只有允许移动时调用
        let rect = this.$refs.canvas.getBoundingClientRect();
        if (ev.touches.length == 1) {
          let obj = {
            x: ev.targetTouches[0].clientX,
            y: ev.targetTouches[0].clientY,
          }
          let canvasX;
          let canvasY;
          canvasX = obj.x - this.$refs.canvas.offsetLeft;
          canvasY = obj.y -  rect.top;
          this.ctx.lineTo(canvasX, canvasY);
          this.ctx.stroke();
        }
      }
    },

    touchEnd() {
      this.existCanvas = true;
      this.canvasMoveUse = false;
    },
 
    getSignImageData() {
      if (this.existCanvas) {
        //判断是否存在手写签名
        let pJson;
        let canvas = document.getElementById("canvas");
        let signature = canvas.toDataURL("image/png");
        let str = signature.replace("data:image/png;base64,", "");
        pJson = {
          type: "image/png;base64",
          data: str
        };
        return pJson;
      }else {
        return false;
      }
      
    },

    //提交
    onSubmit(num) {
      let op = this.openParams;
      if(num == 1) { //自由流程
        // let formData = this.buildFormData;
        let formData = this.getBuildFormData();
        formData.stateId = this.stateId;
        let docId = formData.id,
            id = formData.id;
        let formId =  op.realformId ? op.realformId : formData.formId;
        formData.formId = formId;
        let document = {
          applicationId: this.$root.appid,
          docId: docId,
          document: formData,
          formId: formId,
          id: id,
          nextUserId:this.selectedPerson.id,
          parentId: "",
          templateForm: op.realformId ? op.templateForm : "",
          viewId: ""
        };
        document.attitude = this.attitude;
        this.$api.backFreeFlow(this.$root.appid, docId, document,{
          onSucess: response => {
            let status;
            if(response.data.errcode==0) {
              status = 'SUCCESS';
              this.$emit("closePopup",status)
              Toast({
                  message: response.data.data,
                  iconClass: 'fa fa-check fa-2x'
                });
            }else if(response.data.errcode==4001){
              status = 'ERROR';
              this.$emit("closePopup",status)
              Toast({
                message: response.data.errmsg,
                iconClass: 'fa fa-close fa-2x'
              });
            }
          }
        });
      }else {
        if(this.selectedPerson.value) {
          let pJson = this.getSignImageData()
          // let formData = this.buildFormData;
          let formData = this.getBuildFormData();
          formData.stateId = this.stateId;
          let docId = formData.id,
              id = formData.id;
          let formId = op.realformId ? op.realformId : formData.formId;
          formData.formId = formId;
          let document = {
            applicationId: this.$root.appid,
            currentNodeId: this.currentNodeId,
            docId: docId,
            document: formData,
            flowId: this.flowId,
            formId: formId,
            id: id,
            parentId: "",
            templateForm: op.realformId ? op.templateForm : "",
            viewId: ""
          };
          if (this.nextNodeIds instanceof Array) {
            //多选
            document.nextNodeIds = this.nextNodeIds;
          } else {
            // 单选
            document.nextNodeIds = [];
            document.nextNodeIds.push(this.selectedPerson.id);
          }
          this.$api.runBeforeActionScript(this.$root.appid, this.act.id, document, {
            onSucess: response => {
              if (response.data.errcode == 0) {
                this.submitFlowData(document, pJson);
              }
            }
          });
        }else {
          let status = 'ERROR';
          this.$emit("closePopup",status)
          Toast({
            message: "未选择指定人",
            iconClass: 'fa fa-close fa-2x'
          });
        }
      }
    },

    submitFlowData(allData, pJson) {
      // let formData = this.buildFormData;
      let formData = this.getBuildFormData();
      let appId = this.$root.appid;
      let docId = formData.id;
      let document = allData;
      if (pJson) {
        //手写签名
        document.signatureJson = JSON.stringify(pJson);
      }
      if (this.nextNodeIds instanceof Array) {
        //多选
        document.nextNodeIds = this.nextNodeIds;
      } else {
        // 单选
        document.nextNodeIds = [];
        document.nextNodeIds.push(this.selectedPerson.id);
      }
      document.attitude = this.attitude;
      document.subFlowApprover = []; //多种流程处理方式，此数据为完善未完善
      document.submitTo = []; //多种流程处理方式，此数据为完善未完善
      API.backFlow(appId, docId, document, {
        onSucess: response => {
          let status;
          if(response.data.errcode==0) {
            status = 'SUCCESS';
            this.$emit("closePopup",status)
            Toast({
              message: response.data.data,
              iconClass: 'fa fa-check fa-2x'
            });
            // this.getPDFsignature();
            this.runAfterAction(document);
          }else if(response.data.errcode==4001){
            status = 'ERROR';
            this.$emit("closePopup",status)
            Toast({
              message: response.data.errmsg,
              iconClass: 'fa fa-close fa-2x'
            });
          }
        }
      });
    },

     //执行脚本后
    runAfterAction(document) {
      let _flowType = this.act.flowType; //提交为80
      this.$api.submitRrunAfterActionScript( 
        this.$root.appid,
        this.act.id,
        document,
        document.docId,
        document.viewId,
        document.formId,
        document.templateForm,
        document.parentId,
        '',
        _flowType,
        this.act.type,//act.type
        {
          onSucess: response => {
            if (response.data.errcode == 0 && !response.data.data) {
            }else if(response.data.errcode == 0 && response.data.data) {
              
            }
          }
        }
      );
    },

    getPDFsignature(){
      // let formData = this.buildFormData;
      let formData = this.getBuildFormData();
      let appId = this.$root.appid;
      let docId = formData.id;
      let jsonData = [];
      this.$api.getPDFsignature(appId, docId, jsonData,{
        onSucess: res => {
          debugger
        }
      })
    },
    //保存重新编辑的意见
    updateSuggestion() {
      let opId = this.opinionId,
          opinion = this.inputSecond;
      API.updateOpinion(this.$root.user.id, opId, opinion, {
        onSucess: response => {
          if (response.data.data) {
            let id = response.data.data.id;
            let content = response.data.data.content;
            for (let i = 0; i < this.allSuggests.length; i++) {
              if (this.allSuggests[i].id == id) {
                this.allSuggests[i].content = content;
                this.inputSecond = "";
              }
            }
          }
        }
      });
    },

    //重新编辑意见
    editSuggestion(id, content) {
      this.inputSecond = content;
      this.opinionId = id;
    },

    //删除意见
    deleteSuggestion(id) {
      let opId = id;
      API.deleteComments(this.$root.user.id, opId, {
        onSucess: response => {
          if (response.data.data) {
            let delId = response.data.data.id;
            let allSug = this.allSuggests;
            for (let i = 0; i < allSug.length; i++) {
              if (allSug[i].id == delId) {
                allSug.splice(i, 1);
                i--;
              }
            }
            this.allSuggest = allSug;
          }
        }
      });
    },
    //添加意见
    addSuggestion() {
      let opinion = this.input;
      API.addCommonOpinions(this.$root.user.id, opinion, {
        onSucess: response => {
          let newSuggest = response.data.data;
          this.allSuggests.push(newSuggest);
          this.input = "";
        }
      });
    },
    // 选中的数据放进文本框中
    putData(content) {
      if (this.showDelAndEdit == false) {
        if (!this.attitude) {
          this.attitude = content;
        } else if (this.attitude) {
          this.attitude = this.attitude + "," + content;
        }
      }
    },
    editSuggest() {
      this.showInput = true;
      this.showEditFirst = false;
      this.showEditSecond = true;
      this.showDelAndEdit = true;
    },
    cancelAddSuggest(num) {
      if (num == 1) {
        this.showInput = false;
        this.showDelAndEdit = false;
        this.showEditFirst = false;
      } else if (num == 2) {
        this.showInput = false;
        this.showDelAndEdit = false;
        this.showEditSecond = false;
      }
    },
    showInputBox() {
      this.showInput = true;
      this.showEditFirst = true;
    },

    // //点击常用意见框以外的地方隐藏
    hiddenSuggestbox() {
      this.suggestBoxVisible = false;
    },

    //隐藏小意见框
    hiddenSuggest() {
      this.suggestBoxVisible = !this.suggestBoxVisible;
    },
    changeSignMode() {
      if (this.signMode == "TEXT") {
        this.signMode = "SIGN";
      }
      else {
        this.signMode = "TEXT";
      }
    },

    //获取返回的数据
    getSuggest() {
      this.$api.getCommonOpinions(this.$root.user.id, {
        onSucess: response => {
          this.allSuggests = response.data.data;
        }
      });
    },

    //获取流程退回时的节点
    getBackPanel() {
      //let formData = this.$parent.$parent.$parent.$parent.$parent.buildFormData();
      if(this.act.workFlowType == 1) { //自由流程
        let instanceId;
        for(let i=0; i<this.approvers.length; i++) {
          instanceId = this.approvers[i].instanceId
        }
        this.$api.getHisActorsFreeFlow(this.$root.appid, instanceId, {
          onSucess: response => {
            this.backNodes = response.data.data
          }
        })
      }else if(this.act.workFlowType == 0) {
        // let formData = this.buildFormData; 
        let formData = this.getBuildFormData();
        formData.stateId = this.stateId;
        let docId = formData.id,
            id = formData.id;
        let formId = formData.formId;
        let document = {
          applicationId: this.$root.appid,
          docId: docId,
          document: formData,
          formId: formId,
          id: id,
          parentId: "",
          templateForm: "",
          viewId: ""
        };
        this.$api.getBackPanel(this.$root.appid, docId, document,{
          onSucess: response => {
            this.backNodes = response.data.data.backNodes;
            this.currentNodeId = response.data.data.currentNodeId; //当前节点ID
            this.flowId = response.data.data.flowId;
          }
        });
      }
      
    }
  }
};
</script>